--  Copyright (C) 2000-2001 Simon Wright.
--  All Rights Reserved.
--
--      This program is free software; you can redistribute it
--      and/or modify it under the terms of the Ada Community
--      License which comes with this Library.
--
--      This program is distributed in the hope that it will be
--      useful, but WITHOUT ANY WARRANTY; without even the implied
--      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
--      PURPOSE. See the Ada Community License for more details.
--      You should have received a copy of the Ada Community
--      License with this library, in the file named "Ada Community
--      License" or "ACL". If not, contact the author of this library
--      for a copy.
--

--  $RCSfile: bc-containers-guarded.adb,v $
--  $Revision: 1.5 $
--  $Date: 2001/09/22 17:28:48 $
--  $Author: simon $

package body BC.Containers.Guarded is

   function "=" (Left, Right : in Guarded_Container) return Boolean is
   begin
      return "=" (Base_Container (Left), Base_Container (Right));
   end "=";

   procedure Seize (C : in out Guarded_Container) is
   begin
      BC.Support.Synchronization.Seize (C.Guard.all);
   end Seize;

   procedure Release (C : in out Guarded_Container) is
   begin
      BC.Support.Synchronization.Release (C.Guard.all);
   end Release;

   procedure Initialize (C : in out Guarded_Container) is
   begin
      Initialize (Base_Container (C));
      C.Guard := new Semaphore;
   end Initialize;

   procedure Adjust (C : in out Guarded_Container) is
      --  Consider C := P;
      --  On entry to Adjust, C contains a bitwise copy of P.
   begin
      --  lock P's semaphore
      Seize (C);
      --  make the deep copy
      Adjust (Base_Container (C));
      --  unlock P's semaphore
      Release (C);
      --  create a new semaphore for C
      C.Guard := new Semaphore;
   end Adjust;
   --  XXX Perhaps Adjust should use a lock variable? to ensure it's
   --  released even if an exception occurs.

   procedure Finalize (C : in out Guarded_Container) is
   begin
      BC.Support.Synchronization.Delete (C.Guard);
      Finalize (Base_Container (C));
   end Finalize;

   Empty_Container : Guarded_Container;
   pragma Warnings (Off, Empty_Container);

   function Null_Container return Guarded_Container is
   begin
      return Empty_Container;
   end Null_Container;

end BC.Containers.Guarded;
